from os import listdir, remove, rename
from os.path import isfile, join
import re
from sys import exit

r = re.compile(r'^(.*?)_')

photo_path = "media/photo"
video_path = "media/video"
gif_path = "media/animated_gif"

def clean_dir(dirpath):

    files = [f for f in listdir(dirpath) if isfile(join(dirpath, f))]
    fsize = len(files)
    print('Checking photo files for duplicate IDs...')

    print('Scrubbing away the indices...')

    scrub = []

    for idx, i in enumerate(files):
        filename = i
        match = r.search(i)
        if (match):
            index = i.split('_')[0]
            filename = filename.replace(match.group(0), '')
            scrub.append({'id': index, 'name': filename})
        
    if (len(scrub) == len(files)):
        print('All files were scrubbed!')

    elif (len(scrub) == 0):
        print('All files are already clean! Exiting...')
        return

    elif (len(scrub) <= len(files)):
        print('Some files were scrubbed, others are already clean!')
    print('Scrubbing complete.')

    print('Checking for duplicates...')

    seen = {}
    dups = []

    for i in scrub:
        if i['name'] not in seen:
            seen[i['name']] = 1
        else:
            if seen[i['name']] == 1:
                filename = str(i['id']) + '_' + i['name']
                dups.append(filename)

            seen[i['name']] += 1

    print(f'Found {len(dups)} duplicates.')

    if (len(dups) == 0):
        print('No further action is required, exiting...')
        return

    print('Removing duplicates from directory...')

    c = 0

    for i in dups:
        if i in files:
            path = join(dirpath + '/', i)
            remove(path)
            print(f'{i} removed.')
            c += 1
            

    print(f'All files found? {c == len(dups)}')

    final_size = fsize - c
    print(f'Check directory, there should be {final_size} files! Exiting...')


def rename_files(dirpath):

    files = [f for f in listdir(dirpath) if isfile(join(dirpath, f))]
    fsize = len(files)

    print('Cleaning out the IDs...')

    print('Scrubbing away the indices...')

    scrub = []

    for idx, i in enumerate(files):
        filename = i
        match = r.search(i)
        if (match):
            index = i.split('_')[0]
            filename = filename.replace(match.group(0), '')
            
            # old_path = join(dirpath + '/', i)
            # new_path = join(dirpath + '/', filename)
            # rename(old_path, new_path)
            scrub.append({'id': index, 'name': filename})
        
    if (len(scrub) == len(files)):
        print('All files were scrubbed!')
    elif (len(scrub) == 0):
        print('All files are already clean! Exiting...')
        return
    elif (len(scrub) <= len(files)):        
        print('Some files were scrubbed, others are already clean!')

    print('Scrubbing complete.')

while 1:
    
    choice = input('Which directory do you want to delete? [1]: photo, [2]: video, [3]: gif ')

    if choice == '1':
        clean_dir(photo_path)
        if(input('Do you want to scrub? y/n ').lower() == 'y'):
            rename_files(photo_path)
        break
    elif choice == '2':
        clean_dir(video_path)
        if(input('Do you want to scrub? y/n ').lower() == 'y'):
            rename_files(video_path)
        break
    elif choice == '3':
        clean_dir(gif_path)
        if(input('Do you want to scrub? y/n ').lower() == 'y'):
            rename_files(gif_path)
        break
    else:
        print('Wrong value, try again.')

exit()
        



from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from selenium.common.exceptions import NoSuchElementException, StaleElementReferenceException
from time import sleep
from getpass import getpass
import json
import datetime
import sys

from cryptography.fernet import Fernet

def scrape(tag):
    """

    :param tag: 

    """
    cipher_suite = ""
    DEFAULT_ID = 100
    account = "userinfo.json"
    hasAuth = False
    user = ""
    login = ""
    password = ""

    if (tag == "-f"):
        try:
            with open(account) as f:
                
                print("File found. Reading...") 

                data = json.load(f)
                data_to_write = data

                account_rec = data
                user = account_rec["user"]
                login = account_rec["login"]
                password = account_rec["pass"]
                password = password[2:-1].encode()

                cipher_suite = Fernet(account_rec["key"].encode())

                hasAuth = True
       
        except FileNotFoundError:
            print("File not found. Continuing...")
               

    if not hasAuth:

        key = Fernet.generate_key()
        cipher_suite = Fernet(key)
        
        user = input("What's your twitter username (not display)? ")
        login = input("What's your login? (email/username) ")
        password = getpass("Type in your password. ")
        password = cipher_suite.encrypt(password.encode('utf-8'))

        account_rec = {
            "user": user,
            "login": login,
            "pass": password.decode(),
            "key": key.decode(),
        }

        if input("Do you want me to store your information in a text file? When you run this program again, you can append '-f' to automatically search your account. (y/n) ") == "y":
            try:
                with open(account) as f:
                    
                    print("File found. Writing...") 

                    data = json.load(f)
                    data_to_write = data

                    data_to_write = account_rec
           
            except FileNotFoundError:
                print("File not found. Creating...")
                with open(account, 'w') as f:
                   
                    data_to_write = account_rec
                    
            with open(account, 'w') as outfile:
                json.dump(data_to_write, outfile)
                print("Account info saved! Initiating scrape mode.")


    max_id = input("Type in the maximum number of tweets we'll collect. Otherwise, type n (default: 100). ") 
    if max_id == "n":
        max_id = DEFAULT_ID

    delay = 1  # time to wait on each page load before reading the page
    driver = webdriver.Firefox(executable_path="geckodriver")  # options are Chrome() Firefox() Safari()
    driver.implicitly_wait(10)

    twitter_ids_filename = 'all_ids.json'
    tweet_selector = 'article'
    id_selector = "a[href*=status]"

    user = user.lower()
    ids = []
    errors = []

    MAX_IDS = int(max_id)

    likes_url = f"https://twitter.com/{user}/likes"
    url = "https://twitter.com/login"
    driver.get(url)


    user_selector = 'div.css-1dbjc4n:nth-child(6) > label:nth-child(1) > div:nth-child(1) > div:nth-child(2) > div:nth-child(1) > input:nth-child(1)'
    pass_selector = 'div.css-1dbjc4n:nth-child(7) > label:nth-child(1) > div:nth-child(1) > div:nth-child(2) > div:nth-child(1) > input:nth-child(1)'
    error_selector = 'div.css-901oao:nth-child(3) > span:nth-child(1)'
    scrollElementIntoMiddle = "let viewPortHeight = Math.max(document.documentElement.clientHeight, window.innerHeight || 0);" + "var elementTop = arguments[0].getBoundingClientRect().top;" + "window.scrollBy(0, elementTop-(viewPortHeight/2));";

    sleep(delay)

    try:
        print("Logging in..")
        found_user = driver.find_element_by_css_selector(user_selector)
        found_pass = driver.find_element_by_css_selector(pass_selector)

        found_user.send_keys(login)
        sleep(delay)
        
        print(password)
        password = cipher_suite.decrypt(password)
        password = str(password.decode('utf-8'))
        found_pass.send_keys(password, Keys.ENTER)
     
        sleep(delay)

        try:

            found_error = driver.find_element_by_css_selector(error_selector)
            print("Found error.")
            found_user = driver.find_element_by_css_selector(user_selector)
            # print(found_user)
            found_pass = driver.find_element_by_css_selector(pass_selector)
            # print(found_pass)

            found_user.send_keys(user)
            found_pass.send_keys(password, Keys.ENTER)

        
        except NoSuchElementException:
            print("Successful login!")
        
        print("Checking profile...")
        sleep(delay)
        driver.get(likes_url)
        sleep(delay)
        found_tweets = driver.find_elements_by_css_selector(tweet_selector)
        increment = 5

        timeout = 0
        isEnd = 0
        MAX_END_TIMEOUT = 10
        
        while isEnd <= MAX_END_TIMEOUT:
            
            if len(ids) >= MAX_IDS:
                print("Reached max IDs.")
                break

            print("Looks like there's more. Continuing. . .")


            try:
                while isEnd <= MAX_END_TIMEOUT and len(ids) < MAX_IDS and len(found_tweets) >= increment:
                    print('Loading more tweets, scrolling. . . ')
                  
                  
                    sleep(delay)
                    found_tweets = driver.find_elements_by_css_selector(tweet_selector)
                    
                    numUnique = 0

                    for tweet in found_tweets:
                        
                        try:
                            metadata = tweet.find_element_by_css_selector(id_selector).get_attribute('href').replace("https://twitter.com", "")
                            id = tweet.find_element_by_css_selector(id_selector).get_attribute('href').split('/')[-1]
                
                            if id not in ids:
                                
                                print(f"ID: {id}, x-data: {metadata}")
                                ids.append(id)

                                numUnique += 1
                                isEnd = 0

                                if (len(ids) >= MAX_IDS):
                                    break
                            
                        
                        except StaleElementReferenceException as e:
                            print('Lost element reference.', tweet)

                    print('{} total tweets.'.format(len(ids)))

                    driver.execute_script(scrollElementIntoMiddle, found_tweets[-1])

                    timeout = 0

                    if numUnique == 0:
                         isEnd += 1

                    print("isEnd: " + str(isEnd))


            except NoSuchElementException:
                 print('None found.')

                 if not len(errors) == 0 and ids[-1] == errors[-1]["id"]:
                     timeout += 1

                 print("NonElement Timeout: " + str(timeout))
                 if timeout >= 10:
                     break

                 errors.append({"id": ids[-1], "num": len(ids)})
                 driver.execute_script(scrollElementIntoMiddle, found_tweets[-1])

                    
    except KeyboardInterrupt:
        print('CTRL+C.')

    try:
        with open(twitter_ids_filename) as f:
            
            data = json.load(f)
            data_to_write = data

            for i in ids:
                if i not in data:
                    data_to_write.append(i)

            print('Number of IDs: ', len(ids))
            print('Total Count: ', len(data_to_write))

    except FileNotFoundError:
        with open(twitter_ids_filename, 'w') as f:
            
            data = json.load(f)
            data_to_write = data

            for i in ids:
                if i not in data:
                    data_to_write.append(i)

            print('Number of IDs: ', len(ids))
            print('Total Count: ', len(data_to_write))

    with open(twitter_ids_filename, 'w') as outfile:
        json.dump(data_to_write, outfile)

    print('All done.')
    driver.close()



import os
import requests
from urllib.error import HTTPError
from shutil import copyfileobj
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


def get_video_url(entity, _id, medium):
    data_dict = {'tweet_id': _id, 'media_url': '', 'bitrate': -1, 'type': '', 'medium': medium}

    for variant in entity['video_info']['variants']:
        if 'bitrate' in variant:
            if variant['bitrate'] > data_dict['bitrate']:
                data_dict['media_url'] = variant['url']
                data_dict['bitrate'] = variant['bitrate']
                data_dict['type'] = variant['content_type'][-3:]
    return data_dict


def get_photo_url(entity, _id, medium):
    data_dict = {'tweet_id': _id, 'media_url': '', 'type': '', 'medium': medium}
    data_dict['media_url'] = entity['media_url']
    data_dict['type'] = entity['media_url'][-3:]
    return data_dict


def get_entities(data, _id):
    if 'extended_entities' in data:
        target = data['extended_entities']
    elif 'entities' in data:
        target = data['entities']
    else:
        return {'message': 'No Media Found', 'tweet_id': _id}

    if 'media' in target:
        entities = []
        for ent in target['media']:
            if ent['type'] == 'video':
                entities.append(get_video_url(ent, _id, medium='video'))
            elif ent['type'] == 'photo':
                entities.append(get_photo_url(ent, _id, medium='photo'))
            elif ent['type'] == 'animated_gif':
                entities.append(get_video_url(ent, _id, medium='animated_gif'))
        return entities

    else:
        return {'message': 'No Media Found', 'tweet_id': _id}

def item_retrieve(data_dict):
    try:
        media_dir = os.path.join('media',data_dict['medium'])
        # increment = 0
        while True:
            name = os.path.join(media_dir,'{}.{}'.format(data_dict['tweet_id'], data_dict['type'][-3:]))
            # if os.path.exists(name):
                # increment += 1
            # else:
                # break
            break
        r = requests_retry_session().get(data_dict['media_url'], stream=True)
        with open(name, 'wb') as f:
            copyfileobj(r.raw, f)
        data_dict['media_file'] = name
    except HTTPError:
        data_dict['media_file'] = 'Error - Could Not Retrieve'
        return
    return

def if_no_dir_make(path):
    import os
    try:
        os.makedirs(path)
    except OSError:
        if not os.path.isdir(path):
            raise
    finally:
    	return path

def requests_retry_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(500, 502, 504),
    session=None,
):
    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )
    adapter = HTTPAdapter(max_retries=retry)
    session.mount('http://', adapter)
    session.mount('https://', adapter)
    return session

if __name__ == '__main__':
    pass



import sys
import re
import csv
import json
from tweepy import OAuthHandler, API, Cursor, TweepError
from .functions import get_entities, item_retrieve

def download(filename):

    if input("Ready to download? (y/n) ") == "n":
        sys.exit()

    r = re.compile(r'(?<=\.)csv') 
    ri = re.compile(r'(?<=\.)json')

    ext = ''
    if (r.search(filename)):
        ext = r.search(filename).group(0)

    if (ext == "csv"):
        print("CSV file found.")
        print(f"The script will now load {filename}.")
    elif (ri.search(filename).group(0) == 'json'):
        print("JSON file found.")
        print(f"The script will now load {filename}.")
        ext = "json"

    if input("Continue? ... y/n ").lower() != 'y':
        sys.exit("Quitting.")


    _id = []
    indexes = []

    consumer_key = ""
    consumer_secret = ""
    access_token = ""
    access_token_secret = ""


    try:
        with open("api_keys.json", "r") as f:

            data = json.load(f)

            consumer_key = data["consumer_key"]
            consumer_secret = data["consumer_secret"]
            access_token = data["access_token"]
            access_token_secret = data["access_token_secret"]

    except FileNotFoundError:
    
        print("Missing API information. Ensure an api_keys.json is created with the appropriate information.")
        sys.exit()

    with open(filename, mode="r", encoding="utf-8") as f:
        if ext == "csv":

            csv_reader = csv.DictReader(f)

            for i, row in enumerate(csv_reader):
                indexes.append(i)
                _id.append(row["_id"])
       
        elif ext == "json":

            data = json.load(f)

            for i, item in enumerate(data):
                indexes.append(i)
                _id.append(item)
        
        print(f'Located {len(indexes)} tweets.')
                
        print('Establishing API . . .')
        
        try:
            auth = OAuthHandler(consumer_key, consumer_secret)
            auth.set_access_token(access_token, access_token_secret)
            api = API(auth, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
        except TweepError:
            print('Error establishing API . . .')
            sys.exit("Quitting.")

        print('Checking for media...')
        
        data = []
        for i, t in enumerate(_id):
            try:
                status = api.get_status(t, tweet_mode="extended", include_entities=True)._json
                entities = get_entities(status, t)
                
                if "message" in entities:
                    print(f"No media found, ignoring {t}...")
                    continue
                
                for data_dict in entities:
                    print(f"Found: {t}")
                    data_dict['original_row'] = indexes[i]
                    data_dict['tweet_url'] = f'https://twitter.com/statuses/{str(t)}'
                    data.append(data_dict)
            except TweepError as e:
                data.append({'message':e, 'original_row': i, 'tweet_id': t})
                continue

        num_media = len([x for x in data if 'medium' in x])
        print(f'Retrieved meta-data for {num_media} media items...')

        print("Retrieving media items...")
       
        for row in data:
            if 'medium' in row:
                print(f"Downloading {row} ...")
                item_retrieve(row)
        print(f'Job Complete. Check the "media" folder for your files. Have a nice day!')



from setuptools import setup, find_packages
import os

this = os.path.abspath(os.path.dirname(__file__))
repo = "https://github.com/sweeneyngo/birdysis"

setup(

    # Application name:
    name="birdysis",

    # Version number (initial):
    version="0.1.3",

    # Application author details:
    author="Sweeney Ngo",
    author_email="sweeneyngo@gmail.com",

    # Packages
    packages=find_packages(include=['birdysis', 'birdysis.*']),

    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        ],

    python_requires='>=3.6',

    # Include additional files into the package
    include_package_data=True,

    # Details
    url=repo,


    license="LICENSE",
    description="Retrieves and downloads liked tweets on Twitter.",

    # long_description=Readme,
    # long_description_content_type="text/markdown",

    # Dependent packages (distributions)
    install_requires=[
        "tweepy",
        "selenium",
        "cryptography",

    ],
    entry_points={
        'console_scripts': [
                'birdysis=src.__main__:main',
            ],
        },
)
